/*******************************************************************************
* (c) Copyright 2014 Biricha Digital Power Limited
* FILE          : main.c
* AUTHOR        : Biricha Digital Power Ltd
* PROJECT       : ex3c
* Target System : dsPIC33F
* CREATION DATE : 03/07/2014
* COPYRIGHT     : Copyright Biricha Digital Power Limited 2014. All rights
*                 reserved. Reproduction in whole or part is prohibited without
*                 written consent of the copyright owner. You may receive a free
*                 commercial license by contacting Biricha Digital. Do not
*                 remove or change this copyright notice.
* DESCRIPTION   :
*
* This project demonstrates the use of the Microchip 3p3z controller as the
* compensator for a voltage mode Buck converter with the addition of early
* interrupts and delayed ADC trigger in order to reduce the phase margin
* erosion.
*
* INFO
*
* PWM1H (TP23)     - PWM output
* PWM1L            - Not used
* RB12  (J1 pin 1) - "IsrPin"
* RB8              - "SwitchPin"
*
*******************************************************************************/


/* Device header file */
#include <xc.h>             /* Includes the appropraite device header file    */
#include <stdint.h>         /* For uint16_t definition                        */
#include "smps_control.h"   /* For Microchip's 3p3z controller                */
#include "board.h"          /* Defintions for the board we are using          */
#include "bdp.h"            /* Biricha's re-usable library funcions           */

/******************************************************************************/
/* Global Variable Declaration                                                */
/******************************************************************************/
/* Configuration bits from the config bits generator                          */
_FICD(ICS_PGD2 & JTAGEN_OFF)
_FOSCSEL(FNOSC_FRC)
_FOSC(FCKSM_CSECMD & OSCIOFNC_ON)
_FWDT(FWDTEN_OFF)

/* Set up the controller data structure
 * The library provided by Microchip requires this structure to be declared and
 * its members initialized (we initialize this later on in the main function).
 */
SMPS_3P3Z_T cntrl3P3Z;

/* Set up the controller data structure
 * The library provided by Microchip requires this structure to be declared and
 * its members initialized (we initialize this later on in the main function).
 */
CTRL_SoftStartStop cntrlSS;

/* Declare the arrays to store coefficients, error terms and outputs of the 
 * controller. We tell the compiler to store these in specific sections of 
 * memory
 */
int16_t cntrlACoefficients[3]__attribute__ ((section (".xbss")));
int16_t cntrlBCoefficients[4]__attribute__ ((section (".xbss")));
int16_t cntrlErrorHistory[4] __attribute__ ((space (ymemory), far));
int16_t cntrlControlHistory[3] __attribute__ ((space (ymemory), far));


// Converter Specification
// Vin   = 9V
// Vout  = 3.3V
// Io    = 1.5A
// L0    = 22uH
// DCR   = 120mOhm
// C0    = 323uF
// Resr  = 25mOhm

// Fx    = 15kHz, Pm = 50deg
// Fs    = 350kHz

#define FREQ_HZ  350000
#define B0 (0x7FFF)
#define B1 (0x8A13)
#define B2 (0x8032)
#define B3 (0x761E)
#define A1 (0x353)
#define A2 (0xFF47)
#define A3 (0xFFA7)
#define G  (0x71CE)
#define pre_shift (+0)
#define post_shift (+6)
#define REF (777)
#define DUTY_TICKS_MIN (0)
#define DUTY_TICKS_MAX (2471)




/******************************************************************************/
/* Main Program                                                               */
/*                                                                            */
/* Important: Please see the copyright notice at the top of this file.        */
/******************************************************************************/
int16_t main(void)
{
    /* Configure the system clock, PLL, etc.
     * The configuration within this function was covered yesterday.
     * You can step into this function to see what is being set up.
     * Please don't do this now as it detracts from the objectives of this lab
     */
    SYS_config();

    /* Configure the ADC. Set which channel pair to sample. Set up interrupts.
	 */
    ADC_config(ADC_Early_Interrupt_Enabled, ADC_Order_Odd, INT_Priority_5);

    /* Set pin as output for timing measurement */
    DirIsrPin(0);

    /* Set pin an input for the switch SW1 (used for SS) */
    DirSwitchPin(1);

    /* Configure PWM1 to run at the required switching frequency */
    PWM_init(PWM_freqToTicks(FREQ_HZ));
    PWM1_config();

    /* This will configure the PWM module to trigger the ADC later in the
     * switching period.
     *
     * Note: 200kHz = 5000ns therefore, 5000 - 2750 = 2250ns
     *       350kHz = 2857ns therefore, 2857 - 2750 = 107ns
     *
     *
     *                       <--PERIOD NS-->
     *                       ___             ___
     *        PWM 1: _______|   |___________|   |___________|
     *
     *                  PERIOD NS-2750
     *                      <---->    ____            ____
     *      ADC+ISR: ________________|RB12|__________|RB12|__
     *                           ^        ^
     *          ADC trigger here ^        ^ update CMPDAC1 here
     *
     *
     */
    PWM1_setTrigger(PWM_freqToTicks(FREQ_HZ) - PWM_nsToTicks(2750));

    /* Configure PWM1 with complimentary output but disable the PWM1L pin */
    PWM1_setOutput( PWM_Mode_Complementary,
                    PWM_Pin_Active_High,
                    PWM_Pin_Disable );

    /* Set duty of PWM1H to 0% */
    PWM1_setDutyH( 0 );

    /* Initialize the 3p3z controller with the coefficients and addresses of the
     * arrays used to store the coefficients/states of the controller.
     */
    CTRL_3p3z_init( &cntrl3P3Z,
                    &cntrlACoefficients[0],
                    &cntrlBCoefficients[0],
                    &cntrlControlHistory[0],
                    &cntrlErrorHistory[0],
                    B0, B1, B2, B3,
                    A1, A2, A3,
                    pre_shift,
                    post_shift,
                    G,
                    DUTY_TICKS_MAX,
                    DUTY_TICKS_MIN );

    /* Initialise our soft start / stop structure
     * This is used by the CNTRL_SoftStartStop_Update() function to update the
     * soft start / stop reference value
     */
    CTRL_SoftStartStop_init( &cntrlSS,
                             REF, // Maximum soft start value
                             111, // Number of steps
                             7,   // Increment/decrement per step
                             SS_Mode_Starting );

    /* Enable the PWM module last */
    PWM_enable(true);

    while(1)
    {
        /* Check to see if button connected to RB8 is pressed
         * Apply 10ms of delay for debouncing
         */
        if (GPIO_getB_PushButtonDebounced(8,10))
        {
            cntrlSS.m_Dir ^= 1; /* Swap direction */
        }

        /* Perform update of soft start / stop */
        CTRL_SoftStartStop_Update( &cntrlSS );

        /* If we have soft stopped - disable the PWM */
        if (cntrlSS.m_Ref == 0)
        {
            /* Now shutdown the PWM module gracefully */
            PWM_enable(false);
        } else {
            /* Now enable the PWM module */
            if (PTCONbits.PTEN == 0) PWM_enable(true); 
        }
    }
}

/* This is the interrupt service routine
 */
void __attribute__((__interrupt__, no_auto_psv)) _ADCP0Interrupt()
{
    /* Set output pin high (for timing measurement) */
    SetIsrPin(1);

    /* Clear ADC interrupt flag */
    ADC0_clrInt();

    /* Call 3P3Z compensator
     * This function will carry out the following tasks:
     * 1 - Read the ADC
     * 2 - Calculate the error
     * 3 - Execute the 3p3z controller
     * 4 - Scale the output correctly and bound by min/max limits
     * 5 - Update the duty for PWM1
     */
    SMPS_Controller3P3ZUpdate(&cntrl3P3Z, &ADCBUF1, cntrlSS.m_Ref, &PDC1);

    /* Set output pin low (for timing measurement) */
    SetIsrPin(0);
}

